; -----------------------------------------------------------
;  42_2020 - A TOWEL DAY 2020 SHORT DEMO
;  v0.3 - 23/05/2020 - by Giuseppe Mignogna
; -----------------------------------------------------------
;  C16 assembly source code
; -----------------------------------------------------------
;  Desc: Creates a scrolling rainbow effect for background 
;        and border colors. The code assumes that there is 
;		 a TEDzakker player routine at $2300.
;        Code written in assembly 6502 for the as65 compiler.
;        Load into a plus/4 emulator and run with "sys 8192".
; -----------------------------------------------------------

		org $2000-2
		dw $2000

		lda #$02	; starting color
		sta $d6		; color
		lda #$00	; starting lumi
		sta $d7		; lumi
		lda #$00
		sta $d8		; vpos
		sta $d9		; new vpos 
		sta $da		; flag inc/dec lumi

		jsr $2300	; init TEDzakker player
		
; setup interrupt
		sei  
		lda #lo(irq)
		sta $0314
		lda #hi(irq)
		sta $0315
		lda #$02
		sta $ff0a
		lda $d8
		sta $ff0b
		cli
		
		rts


; RASTER IRQ ROUTINE ---------------------------

irq
		lda $ff09
		sta $ff09 
		
		lda $da
		bne declumi	; if flag != 0, jumps to decrease
		
; increases lumi
		inc $d7
		lda $d7
		cmp #$07	; max lumi
		bne setcolors
		lda #$01	; changes flag to decrement lumi
		sta $da
		jmp setcolors
		
		
; decreases lumi
declumi		
		dec $d7
		lda $d7
		cmp #$00	; min lumi
		bne setcolors
		lda #$00	; changes flag to increment lumi
		sta $da		

; increases lumi		
		inc $d6
		lda $d6
		cmp #$06	; max color  
		bne setcolors
		
		lda #$02	; restores the starting color
		sta $d6

		jsr $2303	; calls TEDzakker player
		
		lda $d9
		adc #$04	; scrolling speed
		sta $d9		; new starting line for the scrolling effect
		sta $d8		; sets the new starting line for the next irq	


; sets the current color and luminance to the pointed raster line		
setcolors
		lda $d7
		asl a
		asl a
		asl a
		asl a
		ora $d6
		sta $ff15		
		sta $ff19

		lda $d8
		adc #$04	; steps to the next line
		sta $d8
		sta $ff0b	; sets the new interrupt line
	
exit		
		jmp $ce0e	; <- comment + uncomment below if m/l only
		
		;pla
		;tay
		;pla
		;tax
		;pla
		;rti
